-----------------------------------------------------------------------------
-- Title           : FT1248 master interface Controller
-- Project         : FT232H
-----------------------------------------------------------------------------
-- File            : FT1248_master.vhd
-- Author          : A.J. Dougan
-- Company         : Future Technology Devices Intl
-- Date Created    : 27-01-2010
-----------------------------------------------------------------------------
-- Description     This will access FT232H chip via the FT1248 bus
--
-----------------------------------------------------------------------------
-- Known issues and omissions: 
--                 
--                 None
-----------------------------------------------------------------------------
-- Copyright 2010 FTDI Ltd. All rights reserved
-----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
 
entity FT1248_master is
--   generic( 
--      cpol_level : integer := 0
--   );
   port(
      reset_n        : in std_logic;
      clk            : in std_logic;
      data_in        : in std_logic_vector(7 downto 0);
      data_av        : in std_logic;
      space_av       : in std_logic;

      spi_width      : in std_logic_vector(3 downto 0);
      spi_cpol       : in std_logic;                   -- clock output level when idle
      spi_bord       : in std_logic;                   -- bit order 1 = LSB first
      spi_fctrl      : in std_logic;                   -- flow control when SS-n not active
      rd_modem_st    : in std_logic;                   -- read modem status
      wr_modem_st    : in std_logic;                   -- write modem status
      flush          : in std_logic;                   -- flush command for FT232H

-- outputs


      spi_sclk       : out std_logic;
      spi_ss_n       : out std_logic;
      spi_miso       : in  std_logic;
      spi_miosio_in  : in  std_logic_vector(7 downto 0);
      spi_miosio_out : out std_logic_vector(7 downto 0);
      spi_miosio_en  : out std_logic;

      done_cmd      : out std_logic;                   -- ack for flush or modem state
      rd            : out std_logic;
      wr            : out std_logic;
      data_out      : out std_logic_vector(7 downto 0)

);
end FT1248_master;

architecture rtl of FT1248_master is


component upcntg 
   generic(
      width : integer
   );
   port(
      clk           : in  std_logic;
      enable        : in  std_logic;
      q             : out std_logic_vector(width-1 downto 0);
      async_reset_n : in  std_logic;
      sync_reset    : in  std_logic
);
   end component;



component dncntlg 
   generic(
      width : integer
   );
   port(
      clk           : in  std_logic;
      enable        : in  std_logic;
      d             : in  std_logic_vector(width-1 downto 0);
      q             : out std_logic_vector(width-1 downto 0);
      load          : in  std_logic;
      async_reset_n : in  std_logic;
      sync_reset    : in  std_logic
);
end component;

constant SPI_8_BIT    : std_logic_vector(3 downto 0) := "1000";
constant SPI_4_BIT    : std_logic_vector(3 downto 0) := "0100";
constant SPI_2_BIT    : std_logic_vector(3 downto 0) := "0010";
constant SPI_1_BIT    : std_logic_vector(3 downto 0) := "0001";

constant CMD_WRITE    : std_logic_vector(3 downto 0) := "0000";
constant CMD_READ     : std_logic_vector(3 downto 0) := "0001";
constant CMD_RD_MDM   : std_logic_vector(3 downto 0) := "0010";
constant CMD_WR_MDM   : std_logic_vector(3 downto 0) := "0011";
constant CMD_FLUSH    : std_logic_vector(3 downto 0) := "0100";


type spstates is (SPIDLE,SPWR1,SPWR2,SPWR3,SPWR4,SPWR5,
                  SPWR6,SPWR7,SPRD1,SPRD2,SPRD3,SPRD4,SPRD5,SPRD6,
                  SPFL1,SPFL2,SPFL3,SPFL4,SPFL5);

signal spist : spstates;

signal int_spi_sclk       : std_logic;
signal int_spi_ss         : std_logic;
signal int_spi_miosio_out : std_logic_vector(7 downto 0);
signal int_spi_miosio_en  : std_logic;

signal spiwrdata : std_logic_vector(7 downto 0);
signal spiwrdata_valid    : std_logic;
signal spi_ld_wr  : std_logic;
signal spirddata : std_logic_vector(7 downto 0);
signal spi_ld_rd  : std_logic;
signal spi_rd             : std_logic;
signal spi_wr             : std_logic;
signal spi_wr_del         : std_logic;
signal shift              : std_logic;
signal rst_bitcount       : std_logic;
signal en_bitcount        : std_logic;
signal bitcount : std_logic_vector(2 downto 0);

signal sel_data           : std_logic;
signal do_read            : std_logic;
signal do_write           : std_logic;
signal done_shift         : std_logic;

signal command  : std_logic_vector(3 downto 0);

signal nack               : std_logic;
signal used_spiwrdata     : std_logic;

signal int_spi_width      : std_logic_vector(3 downto 0);
signal int_spi_cpol       : std_logic;                   -- clock output level when idle
signal int_rd_modem_st    : std_logic;                   -- read modem status
signal int_wr_modem_st    : std_logic;                   -- write modem status
signal int_flush          : std_logic;                   -- flush command for FT232H
signal change             : std_logic;

signal lastwasread        : std_logic;

signal int_done_cmd       : std_logic;
signal cpol_initial_value : std_logic;
signal clk_en : std_logic;

begin

---------------------------------------------------------
-- create cpol start value
---------------------------------------------------------

--CPOL_LOW : if cpol_level = 0 generate
--   cpol_initial_value <= '0';
--end generate CPOL_LOW;
--
--CPOL_HIGH : if cpol_level /= 0 generate
--   cpol_initial_value <= '1';
--end generate CPOL_HIGH;

cpol_initial_value <= '0';

---------------------------------------------------------
-- create data regs
---------------------------------------------------------

regpp:process(reset_n,clk)
begin
if (reset_n='0') then
   spiwrdata <= (others=>'0');
   spiwrdata_valid <= '0';
   spi_wr_del   <= '0';
   clk_en <= '0';
elsif rising_edge(clk) then
   if (spi_ld_wr='1') then
      spiwrdata <= data_in;
   end if;
   if (spi_ld_wr='1') then
      spiwrdata_valid <= '1';
   elsif (used_spiwrdata='1') then
      spiwrdata_valid <= '0';
   end if;
   spi_wr_del <= spi_wr;
--   clk_en <= not clk_en;
   clk_en <= '1';
end if;
end process regpp;


---------------------------------------------------------
-- create state machine for cmdbus interface
---------------------------------------------------------

--
--tbstp : process(reset_n,clk)
--begin
--if (reset_n='0') then
--   sdst <= SDIDLE;
--   int_rd        <= '0';
--   int_wr        <= '0';
--
--elsif((clk'event) AND (clk='1')) then
--   case sdst is
----
--   when SDIDLE => 
--      if (start_run='1') then
--         sdst <= SDCMD1;
--      else
--         sdst <= SDIDLE;
--      end if;
--      int_rd        <= '0';
--      int_wr        <= '0';
--
----------------------------------------------
---- others
----------------------------------------------
--
--   when others =>
--      sdst <= SDIDLE;
--      int_rd        <= '0';
--      int_wr        <= '0';
--
--   end case;
--end if;
--end process tbstp;

do_write <= data_av;
do_read  <= ((space_av and not lastwasread) or 
            (space_av and (not spiwrdata_valid and not data_av))) and not spi_wr and not spi_wr_del;

---nack <= '1' when ((spi_miso/='0') and (bitcount="000") and (shift='0')) else '0';
nack <= '1' when ((spi_miso/='0') and (bitcount="000") and (int_spi_sclk=spi_cpol)) else '0';


---------------------------------------------------------
-- create change signal to stop a burst
---------------------------------------------------------

change <=      (int_spi_width(0) xor spi_width(0)) or
               (int_spi_width(1) xor spi_width(1)) or
               (int_spi_width(2) xor spi_width(2)) or
               (int_spi_width(3) xor spi_width(3)) or
               (int_spi_cpol xor spi_cpol) or
               (int_rd_modem_st xor rd_modem_st) or
               (int_wr_modem_st xor wr_modem_st) or
               (int_flush xor flush);

---------------------------------------------------------
-- create state machine for spi interface
---------------------------------------------------------

spistp : process(reset_n,clk,cpol_initial_value)
begin
if (reset_n='0') then
   spist    <= SPIDLE;
   spi_rd   <= '0';
   spi_wr   <= '0';
   int_spi_sclk      <= '0';
   int_spi_ss        <= '0';
   int_spi_miosio_en <= '0';
   rst_bitcount      <= '0';
   spi_ld_rd         <= '0';
   spi_ld_wr         <= '0';
   shift             <= '0';
   sel_data          <= '0';
   int_spi_miosio_en <= '0';
   used_spiwrdata    <= '0';
   int_spi_width     <= (others=>'0');
   int_spi_cpol      <= cpol_initial_value;
   int_rd_modem_st   <= '0';
   int_wr_modem_st   <= '0';
   int_flush         <= '0';
   lastwasread       <= '0';
   command           <= CMD_READ;
   int_done_cmd      <= '0';

elsif((clk'event) AND (clk='1')) then
   if (clk_en='1') then
   case spist is
--
   when SPIDLE =>
      if (int_spi_ss='1') then -- wait for idle
         spist <= SPIDLE;
         int_spi_ss        <= '0';
         spi_ld_wr         <= '0';
         spi_rd   <= '0';
         command           <= command;
      elsif (int_flush='1') and (spiwrdata_valid='0') then
         spist <= SPFL1;
         int_spi_ss        <= '1';
         spi_ld_wr         <= '0';
         spi_rd   <= '0';
         command           <= CMD_FLUSH;
      elsif (do_read='1') then
         spist <= SPRD1;
         int_spi_ss        <= '1';
         spi_ld_wr         <= '0';
         spi_rd   <= '0';
         if (int_rd_modem_st='1') then
            command           <= CMD_RD_MDM;
         else
            command           <= CMD_READ;
         end if;
      elsif (spiwrdata_valid='1') and (used_spiwrdata='0')  then
         spist <= SPWR1;
         int_spi_ss        <= '1';
         spi_ld_wr         <= '0';
         spi_rd   <= '0';
--         command           <= command;
         if (wr_modem_st='1') then
            command           <= CMD_WR_MDM;
         else
            command           <= CMD_WRITE;
         end if;
      elsif (do_write='1') and (used_spiwrdata='0') then
         spist <= SPWR1;
         int_spi_ss        <= '1';
         spi_ld_wr         <= '1';
         spi_rd   <= '1';
         if (wr_modem_st='1') then
            command           <= CMD_WR_MDM;
         else
            command           <= CMD_WRITE;
         end if;
      else
         spist <= SPIDLE;
         int_spi_ss        <= '0';
         spi_ld_wr         <= '0';
         spi_rd   <= '0';
         command           <= command;
      end if;
--      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= spi_cpol;
--      int_spi_ss        <= '0';
      rst_bitcount      <= '1';
      spi_ld_rd         <= '0';
--      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '0';
      used_spiwrdata    <= '0';
      int_spi_width     <= spi_width;
      int_spi_cpol      <= spi_cpol;
      int_rd_modem_st   <= rd_modem_st and not spi_wr;
      int_wr_modem_st   <= wr_modem_st;
      int_flush         <= flush;
      lastwasread       <= lastwasread;
      int_done_cmd      <= '0';

--------------------------------------------
-- write
--------------------------------------------

-- command

   when SPWR1 => 
      spist <= SPWR2;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '1';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= '0';
      command           <= command;
      int_done_cmd      <= '0';

   when SPWR2 => 
      if (done_shift = '1') then
         spist <= SPWR3;
         shift             <= '0';
         rst_bitcount      <= '1';
      else
         spist <= SPWR2;
         shift             <= not shift;
         rst_bitcount      <= '0';
      end if;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
--      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
--      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '1';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

-- turnaround

   when SPWR3=> 
      spist <= SPWR4;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '0';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

   when SPWR4=> 
      spist <= SPWR5;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '1';
      int_spi_miosio_en <= '1';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

-- data phase

   when SPWR5=> 
--      if (do_write='1') and (do_read='0') and (nack='0') and (change='0') and 
--         (int_spi_width = SPI_8_BIT) then
--         spi_rd   <= '1';
--         spi_ld_wr         <= '1';
--      else
--         spi_rd   <= '0';
--         spi_ld_wr         <= '0';
--      end if;
         spi_rd   <= '0';
         spi_ld_wr         <= '0';
      spist <= SPWR6;
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      shift             <= '0';
      sel_data          <= '1';
      int_spi_miosio_en <= '1';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

   when SPWR6=> 
      if (done_shift = '1') or (nack='1') then
         if (spi_rd='1') then
            spist <= SPWR5;
            shift             <= '0';
            used_spiwrdata    <= not nack;
            rst_bitcount      <= '1';
            spi_ld_wr         <= '1';
            spi_rd            <= '0';
            int_spi_sclk      <= not int_spi_sclk;
         else
            spist <= SPIDLE;
            shift             <= '0';
            used_spiwrdata    <= not nack;
            rst_bitcount      <= '0';
            spi_ld_wr         <= '0';
            spi_rd            <= '0';
            int_spi_sclk      <= spi_cpol;
         end if;   
      else
         spist <= SPWR6;
         shift             <= not shift;
         used_spiwrdata    <= '0';
         rst_bitcount      <= '0';
         spi_ld_wr         <= '0';
         spi_rd            <= '0';
         int_spi_sclk      <= not int_spi_sclk;
      end if;
--      spi_rd   <= '0';
      spi_wr   <= '0';
--      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
--      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
--      spi_ld_wr         <= '0';
--      shift             <= '0';
      sel_data          <= '1';
      int_spi_miosio_en <= '1';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

-- burst data phase

   when SPWR7=> 
      spist <= SPWR6;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '1';
      int_spi_miosio_en <= '1';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

--------------------------------------------
-- read
--------------------------------------------

-- command

   when SPRD1 => 
      spist <= SPRD2;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '1';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= '1';
      command           <= command;
      int_done_cmd      <= '0';

   when SPRD2 => 
      if (done_shift = '1') then
         spist <= SPRD3;
         shift             <= '0';
         rst_bitcount      <= '1';
      else
         spist <= SPRD2;
         shift             <= not shift;
         rst_bitcount      <= '0';
      end if;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
--      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
--      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '1';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

-- turnaround

   when SPRD3=> 
      spist <= SPRD4;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '0';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

   when SPRD4=> 
      spist <= SPRD5;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '1';
      int_spi_miosio_en <= '0';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

-- data phase

   when SPRD5=> 
      if (int_spi_width = SPI_8_BIT) then
         spi_ld_rd         <= '1';
      else
         spi_ld_rd         <= '0';
      end if;   
      spist <= SPRD6;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
--      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '1';
      int_spi_miosio_en <= '0';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

   when SPRD6=> 
      if (done_shift = '1') then
         spist <= SPIDLE;
         if (int_spi_width = SPI_8_BIT) then
            spi_ld_rd         <= '0';
         else
            spi_ld_rd         <= not spi_ld_rd;
         end if;   
         shift             <= '0';
         spi_wr            <= not nack;
         rst_bitcount      <= '0';
         int_rd_modem_st   <= int_rd_modem_st and nack;
         int_spi_sclk      <= spi_cpol;
      elsif (nack='1') then
         spist <= SPIDLE;
         shift             <= '0';
         spi_ld_rd         <= not spi_ld_rd;
         spi_wr            <= '0';
         rst_bitcount      <= '0';
         int_rd_modem_st   <= int_rd_modem_st and nack;
         int_spi_sclk      <= spi_cpol;
      else
         spist <= SPRD6;
         shift             <= not shift;
         spi_ld_rd         <= not spi_ld_rd;
         spi_wr   <= '0';
         rst_bitcount      <= '0';
         int_rd_modem_st   <= int_rd_modem_st;
         int_spi_sclk      <= not int_spi_sclk;
      end if;
      spi_rd   <= '0';
--      spi_wr   <= '0';
--      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
--      rst_bitcount      <= '0';
--      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
--      shift             <= '0';
      sel_data          <= '1';
      int_spi_miosio_en <= '0';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
--      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

--------------------------------------------
-- flush
--------------------------------------------

-- command

   when SPFL1 => 
      spist <= SPFL2;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '1';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

   when SPFL2 => 
      if (done_shift = '1') then
         spist <= SPFL3;
         shift             <= '0';
         rst_bitcount      <= '1';
      else
         spist <= SPFL2;
         shift             <= not shift;
         rst_bitcount      <= '0';
      end if;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
--      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
--      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '1';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

-- turnaround


   when SPFL3=> 
      spist <= SPFL4;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '1';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '0';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';


   when SPFL4=> 
      if (nack='1') then
         spist <= SPIDLE;
      else
         spist <= SPFL5;
      end if;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= not int_spi_sclk;
      int_spi_ss        <= '0';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '0';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;
      int_done_cmd      <= '0';

   when SPFL5=> 
      if (flush='1') then
         spist <= SPFL5;
         int_done_cmd      <= '1';
         int_flush         <= '0';
      else
         spist <= SPIDLE;
         int_done_cmd      <= '0';
         int_flush         <= '0';
      end if;
      spi_rd   <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= int_spi_sclk;
      int_spi_ss        <= '0';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '0';
      used_spiwrdata    <= '0';
      int_spi_width     <= int_spi_width;
      int_spi_cpol      <= int_spi_cpol;
      int_rd_modem_st   <= int_rd_modem_st;
      int_wr_modem_st   <= int_wr_modem_st;
--      int_flush         <= int_flush;
      lastwasread       <= lastwasread;
      command           <= command;

--------------------------------------------
-- others
--------------------------------------------

   when others =>
      spist   <= SPIDLE;
      spi_rd  <= '0';
      spi_wr   <= '0';
      int_spi_sclk      <= '0';
      int_spi_ss        <= '0';
      rst_bitcount      <= '0';
      spi_ld_rd         <= '0';
      spi_ld_wr         <= '0';
      shift             <= '0';
      sel_data          <= '0';
      int_spi_miosio_en <= '0';
      used_spiwrdata    <= '0';
      int_spi_width     <= (others=>'0');
      int_spi_cpol      <= '0';
      int_rd_modem_st   <= '0';
      int_wr_modem_st   <= '0';
      int_flush         <= '0';
      lastwasread       <= '0';
      command           <= CMD_READ;
      int_done_cmd      <= '0';

   end case;
   end if;
end if;
end process spistp;

---------------------------------------------------------
-- create bit counter
---------------------------------------------------------

en_bitcount <= shift and clk_en;

bitcnt1 : upcntg 
   generic map(
      width => 3
   )
   port map(
      clk           => clk,
      enable        => en_bitcount,
      q             => bitcount,
      async_reset_n => reset_n,
      sync_reset    => rst_bitcount
);

---------------------------------------------------------
-- route MIOSIO in data
---------------------------------------------------------

miosio_i_0 : process(clk,reset_n)
begin
if (reset_n='0') then
   spirddata <= (others=>'0');
elsif rising_edge(clk) then
   if (clk_en='1') then
   if (spi_ld_rd='1') then
      if (spi_bord='1') then -- lsb
         if (int_spi_width = SPI_8_BIT) then
            spirddata(0) <= spi_miosio_in(0);
            spirddata(1) <= spi_miosio_in(1);
            spirddata(2) <= spi_miosio_in(2);
            spirddata(3) <= spi_miosio_in(3);
            spirddata(4) <= spi_miosio_in(4);
            spirddata(5) <= spi_miosio_in(5);
            spirddata(6) <= spi_miosio_in(6);
            spirddata(7) <= spi_miosio_in(7);
         elsif (int_spi_width = SPI_4_BIT) then
            if (bitcount="000") then
               spirddata(0) <= spi_miosio_in(0);
               spirddata(1) <= spi_miosio_in(1);
               spirddata(2) <= spi_miosio_in(2);
               spirddata(3) <= spi_miosio_in(3);
            end if;
            if (bitcount="001") then
               spirddata(4) <= spi_miosio_in(0);
               spirddata(5) <= spi_miosio_in(1);
               spirddata(6) <= spi_miosio_in(2);
               spirddata(7) <= spi_miosio_in(3);
            end if;
         elsif (int_spi_width = SPI_2_BIT) then
            if (bitcount="000") then
               spirddata(0) <= spi_miosio_in(0);
               spirddata(1) <= spi_miosio_in(1);
            end if;
            if (bitcount="001") then
               spirddata(2) <= spi_miosio_in(0);
               spirddata(3) <= spi_miosio_in(1);
            end if;
            if (bitcount="010") then
               spirddata(4) <= spi_miosio_in(0);
               spirddata(5) <= spi_miosio_in(1);
            end if;
            if (bitcount="011") then
               spirddata(6) <= spi_miosio_in(0);
               spirddata(7) <= spi_miosio_in(1);
            end if;
         else
            if (bitcount="000") then
               spirddata(0) <= spi_miosio_in(0);
            end if;
            if (bitcount="001") then
               spirddata(1) <= spi_miosio_in(0);
            end if;
            if (bitcount="010") then
               spirddata(2) <= spi_miosio_in(0);
            end if;
            if (bitcount="011") then
               spirddata(3) <= spi_miosio_in(0);
            end if;
            if (bitcount="100") then
               spirddata(4) <= spi_miosio_in(0);
            end if;
            if (bitcount="101") then
               spirddata(5) <= spi_miosio_in(0);
            end if;
            if (bitcount="110") then
               spirddata(6) <= spi_miosio_in(0);
            end if;
            if (bitcount="111") then
               spirddata(7) <= spi_miosio_in(0);
            end if;
         end if;
      else
         if (int_spi_width = SPI_8_BIT) then
            spirddata(0) <= spi_miosio_in(7);
            spirddata(1) <= spi_miosio_in(6);
            spirddata(2) <= spi_miosio_in(5);
            spirddata(3) <= spi_miosio_in(4);
            spirddata(4) <= spi_miosio_in(3);
            spirddata(5) <= spi_miosio_in(2);
            spirddata(6) <= spi_miosio_in(1);
            spirddata(7) <= spi_miosio_in(0);
         elsif (int_spi_width = SPI_4_BIT) then
            if (bitcount="000") then
               spirddata(4) <= spi_miosio_in(3);
               spirddata(5) <= spi_miosio_in(2);
               spirddata(6) <= spi_miosio_in(1);
               spirddata(7) <= spi_miosio_in(0);
            end if;
            if (bitcount="001") then
               spirddata(0) <= spi_miosio_in(3);
               spirddata(1) <= spi_miosio_in(2);
               spirddata(2) <= spi_miosio_in(1);
               spirddata(3) <= spi_miosio_in(0);
            end if;
         elsif (int_spi_width = SPI_2_BIT) then
            if (bitcount="000") then
               spirddata(6) <= spi_miosio_in(1);
               spirddata(7) <= spi_miosio_in(0);
            end if;
            if (bitcount="001") then
               spirddata(4) <= spi_miosio_in(1);
               spirddata(5) <= spi_miosio_in(0);
            end if;
            if (bitcount="010") then
               spirddata(2) <= spi_miosio_in(1);
               spirddata(3) <= spi_miosio_in(0);
            end if;
            if (bitcount="011") then
               spirddata(0) <= spi_miosio_in(1);
               spirddata(1) <= spi_miosio_in(0);
            end if;
         else
            if (bitcount="000") then
               spirddata(7) <= spi_miosio_in(0);
            end if;
            if (bitcount="001") then
               spirddata(6) <= spi_miosio_in(0);
            end if;
            if (bitcount="010") then
               spirddata(5) <= spi_miosio_in(0);
            end if;
            if (bitcount="011") then
               spirddata(4) <= spi_miosio_in(0);
            end if;
            if (bitcount="100") then
               spirddata(3) <= spi_miosio_in(0);
            end if;
            if (bitcount="101") then
               spirddata(2) <= spi_miosio_in(0);
            end if;
            if (bitcount="110") then
               spirddata(1) <= spi_miosio_in(0);
            end if;
            if (bitcount="111") then
               spirddata(0) <= spi_miosio_in(0);
            end if;
         end if;
      end if;
   end if;
   end if;
end if;
end process miosio_i_0;

---------------------------------------------------------
-- route MIOSIO out data
---------------------------------------------------------

mxmiosio_o : process(command,spiwrdata,bitcount,int_spi_width,sel_data,spi_bord,int_spi_sclk,spi_cpol)
begin
if (spi_bord='1') then -- lsb
   if (int_spi_width = SPI_8_BIT) then
      if (sel_data='1') then
         int_spi_miosio_out(0) <= spiwrdata(0);
         int_spi_miosio_out(1) <= spiwrdata(1);
         int_spi_miosio_out(2) <= spiwrdata(2);
         int_spi_miosio_out(3) <= spiwrdata(3);
         int_spi_miosio_out(4) <= spiwrdata(4);
         int_spi_miosio_out(5) <= spiwrdata(5);
         int_spi_miosio_out(6) <= spiwrdata(6);
         int_spi_miosio_out(7) <= spiwrdata(7);
      else
         int_spi_miosio_out(0) <= command(3);
         int_spi_miosio_out(1) <= '0';
         int_spi_miosio_out(2) <= '0';
         int_spi_miosio_out(3) <= command(2);
         int_spi_miosio_out(4) <= '0';
         int_spi_miosio_out(5) <= command(1);
         int_spi_miosio_out(6) <= command(0);
         int_spi_miosio_out(7) <= '1';
      end if;
      --done_shift <= '1';
      done_shift <= not (int_spi_sclk xor spi_cpol);
   elsif (int_spi_width = SPI_4_BIT) then
      if (sel_data='1') then
         if (bitcount="000") then
            int_spi_miosio_out(0) <= spiwrdata(0);
            int_spi_miosio_out(1) <= spiwrdata(1);
            int_spi_miosio_out(2) <= spiwrdata(2);
            int_spi_miosio_out(3) <= spiwrdata(3);
            done_shift <= '0';
         else
            int_spi_miosio_out(0) <= spiwrdata(4);
            int_spi_miosio_out(1) <= spiwrdata(5);
            int_spi_miosio_out(2) <= spiwrdata(6);
            int_spi_miosio_out(3) <= spiwrdata(7);
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      else
         if (bitcount="000") then
            int_spi_miosio_out(0) <= command(3);
            int_spi_miosio_out(1) <= '0';
            int_spi_miosio_out(2) <= '0';
            int_spi_miosio_out(3) <= command(2);
            done_shift <= '0';
         else
            int_spi_miosio_out(0) <= '0';
            int_spi_miosio_out(1) <= command(1);
            int_spi_miosio_out(2) <= command(0);
            int_spi_miosio_out(3) <= '1';
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      end if;
      int_spi_miosio_out(7 downto 4) <= "1111";
   elsif (int_spi_width = SPI_2_BIT) then
      if (sel_data='1') then
         if (bitcount="000") then
            int_spi_miosio_out(0) <= spiwrdata(0);
            int_spi_miosio_out(1) <= spiwrdata(1);
            done_shift <= '0';
         elsif (bitcount="001") then
            int_spi_miosio_out(0) <= spiwrdata(2);
            int_spi_miosio_out(1) <= spiwrdata(3);
            done_shift <= '0';
         elsif (bitcount="010") then
            int_spi_miosio_out(0) <= spiwrdata(4);
            int_spi_miosio_out(1) <= spiwrdata(5);
            done_shift <= '0';
         else
            int_spi_miosio_out(0) <= spiwrdata(6);
            int_spi_miosio_out(1) <= spiwrdata(7);
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      else
         if (bitcount="000") then
            int_spi_miosio_out(0) <= command(3);
            int_spi_miosio_out(1) <= '0';
            done_shift <= '0';
         elsif (bitcount="001") then
            int_spi_miosio_out(0) <= '0';
            int_spi_miosio_out(1) <= command(2);
            done_shift <= '0';
         elsif (bitcount="010") then
            int_spi_miosio_out(0) <= '0';
            int_spi_miosio_out(1) <= command(1);
            done_shift <= '0';
         else
            int_spi_miosio_out(0) <= command(0);
            int_spi_miosio_out(1) <= '1';
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      end if;
      int_spi_miosio_out(7 downto 2) <= "111111";
   else
      if (sel_data='1') then
         if (bitcount="000") then
            int_spi_miosio_out(0) <= spiwrdata(0);
            done_shift <= '0';
         elsif (bitcount="001") then
            int_spi_miosio_out(0) <= spiwrdata(1);
            done_shift <= '0';
         elsif (bitcount="010") then
            int_spi_miosio_out(0) <= spiwrdata(2);
            done_shift <= '0';
         elsif (bitcount="011") then
            int_spi_miosio_out(0) <= spiwrdata(3);
            done_shift <= '0';
         elsif (bitcount="100") then
            int_spi_miosio_out(0) <= spiwrdata(4);
            done_shift <= '0';
         elsif (bitcount="101") then
            int_spi_miosio_out(0) <= spiwrdata(5);
            done_shift <= '0';
         elsif (bitcount="110") then
            int_spi_miosio_out(0) <= spiwrdata(6);
            done_shift <= '0';
         else --elsif (bitcount="111") then
            int_spi_miosio_out(0) <= spiwrdata(7);
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      else
         if (bitcount="000") then
            int_spi_miosio_out(0) <= command(3);
            done_shift <= '0';
         elsif (bitcount="001") then
            int_spi_miosio_out(0) <= '0';
            done_shift <= '0';
         elsif (bitcount="010") then
            int_spi_miosio_out(0) <= '0';
            done_shift <= '0';
         elsif (bitcount="011") then
            int_spi_miosio_out(0) <= command(2);
            done_shift <= '0';
         elsif (bitcount="100") then
            int_spi_miosio_out(0) <= '0';
            done_shift <= '0';
         elsif (bitcount="101") then
            int_spi_miosio_out(0) <= command(1);
            done_shift <= '0';
         elsif (bitcount="110") then
            int_spi_miosio_out(0) <= command(0);
            done_shift <= '1';
         else --elsif (bitcount="111") then
            int_spi_miosio_out(0) <= '1';
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      end if;
      int_spi_miosio_out(7 downto 1) <= "1111111";
   end if;
else
   if (int_spi_width = SPI_8_BIT) then
      if (sel_data='1') then
         int_spi_miosio_out(0) <= spiwrdata(7);
         int_spi_miosio_out(1) <= spiwrdata(6);
         int_spi_miosio_out(2) <= spiwrdata(5);
         int_spi_miosio_out(3) <= spiwrdata(4);
         int_spi_miosio_out(4) <= spiwrdata(3);
         int_spi_miosio_out(5) <= spiwrdata(2);
         int_spi_miosio_out(6) <= spiwrdata(1);
         int_spi_miosio_out(7) <= spiwrdata(0);
      else
         int_spi_miosio_out(0) <= command(3);
         int_spi_miosio_out(1) <= '0';
         int_spi_miosio_out(2) <= '0';
         int_spi_miosio_out(3) <= command(2);
         int_spi_miosio_out(4) <= '0';
         int_spi_miosio_out(5) <= command(1);
         int_spi_miosio_out(6) <= command(0);
         int_spi_miosio_out(7) <= '1';
      end if;
--      done_shift <= '1';
      done_shift <= not (int_spi_sclk xor spi_cpol);
   elsif (int_spi_width = SPI_4_BIT) then
      if (sel_data='1') then
         if (bitcount="000") then
            int_spi_miosio_out(0) <= spiwrdata(7);
            int_spi_miosio_out(1) <= spiwrdata(6);
            int_spi_miosio_out(2) <= spiwrdata(5);
            int_spi_miosio_out(3) <= spiwrdata(4);
            done_shift <= '0';
         else
            int_spi_miosio_out(0) <= spiwrdata(3);
            int_spi_miosio_out(1) <= spiwrdata(2);
            int_spi_miosio_out(2) <= spiwrdata(1);
            int_spi_miosio_out(3) <= spiwrdata(0);
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      else
         if (bitcount="000") then
            int_spi_miosio_out(0) <= command(3);
            int_spi_miosio_out(1) <= '0';
            int_spi_miosio_out(2) <= '0';
            int_spi_miosio_out(3) <= command(2);
            done_shift <= '0';
         else
            int_spi_miosio_out(0) <= '0';
            int_spi_miosio_out(1) <= command(1);
            int_spi_miosio_out(2) <= command(0);
            int_spi_miosio_out(3) <= '1';
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      end if;
      int_spi_miosio_out(7 downto 4) <= "1111";
   elsif (int_spi_width = SPI_2_BIT) then
      if (sel_data='1') then
         if (bitcount="000") then
            int_spi_miosio_out(0) <= spiwrdata(7);
            int_spi_miosio_out(1) <= spiwrdata(6);
            done_shift <= '0';
         elsif (bitcount="001") then
            int_spi_miosio_out(0) <= spiwrdata(5);
            int_spi_miosio_out(1) <= spiwrdata(4);
            done_shift <= '0';
         elsif (bitcount="010") then
            int_spi_miosio_out(0) <= spiwrdata(3);
            int_spi_miosio_out(1) <= spiwrdata(2);
            done_shift <= '0';
         else
            int_spi_miosio_out(0) <= spiwrdata(1);
            int_spi_miosio_out(1) <= spiwrdata(0);
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      else
         if (bitcount="000") then
            int_spi_miosio_out(0) <= command(3);
            int_spi_miosio_out(1) <= '0';
            done_shift <= '0';
         elsif (bitcount="001") then
            int_spi_miosio_out(0) <= '0';
            int_spi_miosio_out(1) <= command(2);
            done_shift <= '0';
         elsif (bitcount="010") then
            int_spi_miosio_out(0) <= '0';
            int_spi_miosio_out(1) <= command(1);
            done_shift <= '0';
         else
            int_spi_miosio_out(0) <= command(0);
            int_spi_miosio_out(1) <= '1';
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      end if;
      int_spi_miosio_out(7 downto 2) <= "111111";
   else
      if (sel_data='1') then
         if (bitcount="000") then
            int_spi_miosio_out(0) <= spiwrdata(7);
            done_shift <= '0';
         elsif (bitcount="001") then
            int_spi_miosio_out(0) <= spiwrdata(6);
            done_shift <= '0';
         elsif (bitcount="010") then
            int_spi_miosio_out(0) <= spiwrdata(5);
            done_shift <= '0';
         elsif (bitcount="011") then
            int_spi_miosio_out(0) <= spiwrdata(4);
            done_shift <= '0';
         elsif (bitcount="100") then
            int_spi_miosio_out(0) <= spiwrdata(3);
            done_shift <= '0';
         elsif (bitcount="101") then
            int_spi_miosio_out(0) <= spiwrdata(2);
            done_shift <= '0';
         elsif (bitcount="110") then
            int_spi_miosio_out(0) <= spiwrdata(1);
            done_shift <= '0';
         else --elsif (bitcount="111") then
            int_spi_miosio_out(0) <= spiwrdata(0);
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      else
         if (bitcount="000") then
            int_spi_miosio_out(0) <= command(3);
            done_shift <= '0';
         elsif (bitcount="001") then
            int_spi_miosio_out(0) <= '0';
            done_shift <= '0';
         elsif (bitcount="010") then
            int_spi_miosio_out(0) <= '0';
            done_shift <= '0';
         elsif (bitcount="011") then
            int_spi_miosio_out(0) <= command(2);
            done_shift <= '0';
         elsif (bitcount="100") then
            int_spi_miosio_out(0) <= '0';
            done_shift <= '0';
         elsif (bitcount="101") then
            int_spi_miosio_out(0) <= command(1);
            done_shift <= '0';
         elsif (bitcount="110") then
            int_spi_miosio_out(0) <= command(0);
            done_shift <= '1';
         else --elsif (bitcount="111") then
            int_spi_miosio_out(0) <= '1';
            done_shift <= '1';
--            done_shift <= not (int_spi_sclk xor spi_cpol);
         end if;
      end if;
      int_spi_miosio_out(7 downto 1) <= "1111111";
   end if;
end if;
end process mxmiosio_o;

---------------------------------------------------------
-- assign outputs
---------------------------------------------------------


rd <= spi_rd;
wr <= spi_wr_del;

spi_sclk       <= int_spi_sclk       ;
spi_ss_n       <= not int_spi_ss     ;
spi_miosio_out <= int_spi_miosio_out ;
spi_miosio_en  <= int_spi_miosio_en  ;
data_out       <= spirddata;

done_cmd       <= int_done_cmd;

end rtl;

