-----------------------------------------------------------------------------
-- Title           : FT1248 master interface Controller
-- Project         : FT232H
-----------------------------------------------------------------------------
-- File            : FT1248_master_cont_cont.vhd
-- Author          : A.J. Dougan
-- Company         : Future Technology Devices Intl
-- Date Created    : 27-01-2010
-----------------------------------------------------------------------------
-- Description     This will control the FT1248_master block by decoding a wrapper
--                 from the host PC. 
-----------------------------------------------------------------------------
-- Known issues and omissions: 
--                 
--                 None
-----------------------------------------------------------------------------
-- Copyright 2010 FTDI Ltd. All rights reserved
-----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
 
entity FT1248_master_cont is
   generic( 
      cpol_level : integer := 0;
      loopback   : integer := 0
   );
   port(
      reset_n        : in std_logic;
      clk            : in std_logic;
      data_in        : in std_logic_vector(7 downto 0);
      data_av        : in std_logic;
      space_av       : in std_logic;

-- outputs

      spi_sclk       : out std_logic;
      spi_ss_n       : out std_logic;
      spi_miso       : in  std_logic;
      spi_miosio_in  : in  std_logic_vector(7 downto 0);
      spi_miosio_out : out std_logic_vector(7 downto 0);
      spi_miosio_en  : out std_logic;

      ext_rd         : out std_logic;
      ext_wr         : out std_logic;
      data_out       : out std_logic_vector(7 downto 0)

);
end FT1248_master_cont;

architecture rtl of FT1248_master_cont is


component upcntg 
   generic(
      width : integer
   );
   port(
      clk           : in  std_logic;
      enable        : in  std_logic;
      q             : out std_logic_vector(width-1 downto 0);
      async_reset_n : in  std_logic;
      sync_reset    : in  std_logic
);
   end component;



component dncntlg 
   generic(
      width : integer
   );
   port(
      clk           : in  std_logic;
      enable        : in  std_logic;
      d             : in  std_logic_vector(width-1 downto 0);
      q             : out std_logic_vector(width-1 downto 0);
      load          : in  std_logic;
      async_reset_n : in  std_logic;
      sync_reset    : in  std_logic
);
end component;

component FT1248_master 
--   generic( cpol_level : integer := 0
--   );
   port(
      reset_n        : in std_logic;
      clk            : in std_logic;
      data_in        : in std_logic_vector(7 downto 0);
      data_av        : in std_logic;
      space_av       : in std_logic;

      spi_width      : in std_logic_vector(3 downto 0);
      spi_cpol       : in std_logic;                   -- clock output level when idle
      spi_bord       : in std_logic;                   -- bit order 1 = LSB first
      spi_fctrl      : in std_logic;                   -- flow control when SS-n not active
      rd_modem_st    : in std_logic;                   -- read modem status
      wr_modem_st    : in std_logic;                   -- write modem status
      flush          : in std_logic;                   -- flush command for FT232H

-- outputs


      spi_sclk       : out std_logic;
      spi_ss_n       : out std_logic;
      spi_miso       : in  std_logic;
      spi_miosio_in  : in  std_logic_vector(7 downto 0);
      spi_miosio_out : out std_logic_vector(7 downto 0);
      spi_miosio_en  : out std_logic;

      done_cmd      : out std_logic;                   -- ack for flush or modem state
      rd            : out std_logic;
      wr            : out std_logic;
      data_out      : out std_logic_vector(7 downto 0)

);
end component;

component sync_fifo 
   port(                                 
      reset_n   : in  std_logic;          -- active low reset
--
      s_clk    : in  std_logic;           -- clock from data source
      s_wr     : in  std_logic;           -- source write
      s_txe    : out std_logic;           -- source space available
      s_dbin   : in  std_logic_vector(7 downto 0);  -- source data in
--
      d_clk    : in  std_logic;           -- clock from data destination
      d_rd     : in  std_logic;           -- destination read
      d_rxf    : out std_logic;           -- destination data available
      d_dbout  : out std_logic_vector(7 downto 0)  -- destination data out
   );
end component;


constant CMD_WRITE    : std_logic_vector(3 downto 0) := "0000";
constant CMD_READ     : std_logic_vector(3 downto 0) := "0001";
constant CMD_RD_MDM   : std_logic_vector(3 downto 0) := "0010";
constant CMD_WR_MDM   : std_logic_vector(3 downto 0) := "0011";
constant CMD_FLUSH    : std_logic_vector(3 downto 0) := "0100";
constant CMD_SETUP    : std_logic_vector(3 downto 0) := "0101";


type spstates is (SPIDLE,SPWR1,SPWR2,SPWR3,SPCMD1,SPCMD2,
                  SPWRM1,SPWRM2,SPRDM1,SPRDM2,SPSET1,SPSET2,SPFLSH1);

signal spist : spstates;

signal spi_width      : std_logic_vector(3 downto 0);
signal spi_cpol       : std_logic;                   -- clock output level when idle
signal spi_bord       : std_logic;                   -- bit order 1 = LSB first
signal spi_fctrl      : std_logic;                   -- flow control when SS-n not active
signal spi_ld_setup   : std_logic; 
signal rd_modem_st    : std_logic;                   -- read modem status
signal wr_modem_st    : std_logic;                   -- write modem status
signal flush          : std_logic;                   -- flush command for FT232H
signal spi_data_av    : std_logic; 
signal spi_space_av   : std_logic; 

signal command        : std_logic_vector(3 downto 0);
signal ld_command     : std_logic; 
signal byte_count     : std_logic_vector(15 downto 0);
signal act_count      : std_logic_vector(15 downto 0);
signal ld_cnt_low     : std_logic; 
signal ld_cnt_high    : std_logic; 
signal cnt_limit      : std_logic; 

signal sel_data       : std_logic; 
signal done_cmd       : std_logic; 
signal rd             : std_logic; 
signal wr             : std_logic; 
signal rst_bytecount  : std_logic; 
signal spi_rd         : std_logic; 
signal spi_wr         : std_logic; 
signal enable         : std_logic;

--
-- modify these if using loopback mode to set the mode you want it to work in
--
constant initial_spi_width  : std_logic_vector(3 downto 0) := "0000";
constant initial_spi_cpol   : std_logic := '1';
constant initial_spi_bord   : std_logic := '0';
constant initial_spi_fctrl  : std_logic := '0';

signal holding_reg : std_logic_vector(7 downto 0);
signal int_data_out : std_logic_vector(7 downto 0);
signal int_data_full : std_logic;

begin

---------------------------------------------------------
-- data buffers to / from Morphic HS chip
---------------------------------------------------------


---------------------------------------------------------
-- create data regs
---------------------------------------------------------

regpp:process(reset_n,clk)
begin
if (reset_n='0') then
   spi_width  <= initial_spi_width;
   spi_cpol   <= initial_spi_cpol;
   spi_bord   <= initial_spi_bord;
   spi_fctrl  <= initial_spi_fctrl;
   command    <= (others=>'0');
   byte_count(7 downto 0) <= (others=>'0');
elsif rising_edge(clk) then
   if (spi_ld_setup='1') then
      spi_width  <= data_in(3 downto 0);
      spi_cpol   <= data_in(4);
      spi_bord   <= data_in(5);
      spi_fctrl  <= data_in(6);
   end if;
   if (ld_command='1') then
      command  <= data_in(3 downto 0);
   end if;
   if (ld_cnt_low='1') then
      byte_count(7 downto 0)  <= data_in;
   end if;
end if;
end process regpp;


NON_LOOPBACK : if loopback /= 1 generate

---------------------------------------------------------
-- pull in spi master
---------------------------------------------------------

spi_data_av  <= sel_data and data_av;
spi_space_av <= space_av;

mast1 : FT1248_master 
--   generic map( 
--   cpol_level => cpol_level
--   )
   port map(
      reset_n        =>      reset_n        ,
      clk            =>      clk            ,
      data_in        =>      data_in        ,
      data_av        =>      spi_data_av    ,
      space_av       =>      spi_space_av   ,
      spi_width      =>      spi_width      ,
      spi_cpol       =>      spi_cpol       ,
      spi_bord       =>      spi_bord       ,
      spi_fctrl      =>      spi_fctrl      ,
      rd_modem_st    =>      rd_modem_st    ,
      wr_modem_st    =>      wr_modem_st    ,
      flush          =>      flush          ,

      spi_sclk       =>      spi_sclk       ,
      spi_ss_n       =>      spi_ss_n       ,
      spi_miso       =>      spi_miso       ,
      spi_miosio_in  =>      spi_miosio_in  ,
      spi_miosio_out =>      spi_miosio_out ,
      spi_miosio_en  =>      spi_miosio_en  ,

      done_cmd       =>      done_cmd      ,
      rd             =>      rd            ,
      wr             =>      wr            ,
      data_out       =>      data_out      
   );

---------------------------------------------------------
-- byte counter for writes
---------------------------------------------------------

byte_count(15 downto 8)  <= data_in;
enable <= sel_data and rd;

bcnt1 : dncntlg 
   generic map(
      width => 16
   )
   port map(
      clk           => clk,
      enable        => enable,
      d             => byte_count,
      q             => act_count,
      load          => ld_cnt_high,
      async_reset_n => reset_n,
      sync_reset    => rst_bytecount
);

cnt_limit <= '1' when (act_count="0000000000000000") else '0';

---------------------------------------------------------
-- create state machine for spi interface
---------------------------------------------------------

spistp : process(reset_n,clk)
begin
if (reset_n='0') then
   spist        <= SPIDLE;
   spi_rd       <= '0';
   spi_wr       <= '0';
   rst_bytecount <= '0';
   sel_data     <= '0';
   ld_command   <= '0';
   ld_cnt_low   <= '0';
   ld_cnt_high  <= '0';
   spi_ld_setup <= '0';
   rd_modem_st  <= '0';
   wr_modem_st  <= '0';
   flush        <= '0';

elsif((clk'event) AND (clk='1')) then
   case spist is
--
   when SPIDLE =>
      if (data_av='1') and (spi_rd='0') then -- get command
         spist <= SPCMD1;
         ld_command   <= '1';
         spi_rd       <= '1';
      else
         spist <= SPIDLE;
         ld_command   <= '0';
         spi_rd       <= '0';
      end if;
--      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
--      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';

--------------------------------------------
-- command
--------------------------------------------
   when SPCMD1 =>
      spist <= SPCMD2;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';
--
   when SPCMD2 =>
      if (command=CMD_SETUP) then
         spist <= SPSET1;
      elsif (command=CMD_RD_MDM) then
         spist <= SPRDM1;
      elsif (command=CMD_WR_MDM) then
         spist <= SPWRM1;
      elsif (command=CMD_FLUSH) then
         spist <= SPFLSH1;
      elsif (command=CMD_WRITE) then
         spist <= SPWR1;
      else
         spist <= SPIDLE;
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';

--------------------------------------------
-- setup
--------------------------------------------
   when SPSET1 =>
      if (data_av='1') then -- get setup data
         spist <= SPIDLE;
         spi_rd       <= '1';
         spi_ld_setup <= '1';
      else
         spist <= SPSET1;
         spi_rd       <= '0';
         spi_ld_setup <= '0';
      end if;
--      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
--      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';

--------------------------------------------
-- flush
--------------------------------------------
   when SPFLSH1 =>
      if (done_cmd='1') then 
         spist     <= SPIDLE;
         flush     <= '0';
      else
         spist     <= SPFLSH1;
         flush     <= '1';
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
--      flush        <= '0';

--------------------------------------------
-- wr modem stat
--------------------------------------------
   when SPWRM1 =>
      if (data_av='1') then -- modem data avail
         spist <= SPWRM2;
         wr_modem_st  <= '1';
         sel_data     <= '1';
      else
         spist <= SPWRM1;
         wr_modem_st  <= '0';
         sel_data     <= '0';
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
--      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
--      wr_modem_st  <= '0';
      flush        <= '0';
--
   when SPWRM2 =>
      if (rd='1') then -- get setup data
         spist <= SPIDLE;
         sel_data     <= '0';
         wr_modem_st  <= '0';
         spi_rd       <= '1';
      else
         spist <= SPWRM2;
         sel_data     <= '1';
         wr_modem_st  <= '1';
         spi_rd       <= '0';
      end if;
--      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
--      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
--      wr_modem_st  <= '0';
      flush        <= '0';

--------------------------------------------
-- rd modem stat
--------------------------------------------
   when SPRDM1 =>
      if (space_av='1') then -- modem data avail
         spist <= SPRDM2;
         rd_modem_st  <= '1';
      else
         spist <= SPRDM1;
         rd_modem_st  <= '0';
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
--      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';
--
   when SPRDM2 =>
      if (wr='1') then -- get setup data
         spist <= SPIDLE;
         rd_modem_st  <= '0';
      else
         spist <= SPRDM2;
         rd_modem_st  <= '1';
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
--      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';

--------------------------------------------
-- wr modem stat
--------------------------------------------
   when SPWR1 =>
      if (data_av='1') then -- cnt low data avail
         spist <= SPWR2;
         ld_cnt_low   <= '1';
         spi_rd       <= '1';
      else
         spist <= SPWR1;
         ld_cnt_low   <= '0';
         spi_rd       <= '0';
      end if;
--      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '1';
      sel_data     <= '0';
      ld_command   <= '0';
--      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';
--
   when SPWR2 =>
      if (data_av='1') and (spi_rd='0') then -- cnt high data avail
         spist <= SPWR3;
         spi_rd       <= '1';
         ld_cnt_high  <= '1';
      else
         spist <= SPWR2;
         spi_rd       <= '0';
         ld_cnt_high  <= '0';
      end if;
--      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
--      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';
--
   when SPWR3 =>
      if (rd='1') and (cnt_limit='1') then -- wait for data to be consumed
         spist <= SPIDLE;
         sel_data     <= '0';
      else
         spist <= SPWR3;
         sel_data     <= '1';
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
--      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';


--------------------------------------------
-- others
--------------------------------------------

   when others =>
      spist        <= SPIDLE;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';

   end case;
end if;
end process spistp;



---------------------------------------------------------
-- assign outputs
---------------------------------------------------------


ext_rd <= spi_rd or (sel_data and rd);
ext_wr <= spi_wr or wr;

end generate NON_LOOPBACK;





LOOPBACK_MODE : if loopback = 1 generate

---------------------------------------------------------
-- creatye loopback interface
---------------------------------------------------------

lpbckp : process(reset_n,clk)
begin
if (reset_n='0') then
   holding_reg <= (others=>'0');
   int_data_full <= '0';
elsif rising_edge(clk) then
   if (wr='1') then
      holding_reg <= int_data_out;
   end if;   
   if (wr='1') then
      int_data_full <= '1';
   elsif (rd='1') then   
      int_data_full <= '0';
   end if;   
end if;
end process lpbckp;

---------------------------------------------------------
-- pull in spi master
---------------------------------------------------------

spi_data_av  <= int_data_full;
spi_space_av <= not int_data_full;

mast1 : FT1248_master 
--   generic map( 
--   cpol_level => cpol_level
--   )
   port map(
      reset_n        =>      reset_n        ,
      clk            =>      clk            ,
      data_in        =>      holding_reg    ,
      data_av        =>      spi_data_av    ,
      space_av       =>      spi_space_av   ,
      spi_width      =>      spi_width      ,
      spi_cpol       =>      spi_cpol       ,
      spi_bord       =>      spi_bord       ,
      spi_fctrl      =>      spi_fctrl      ,
      rd_modem_st    =>      rd_modem_st    ,
      wr_modem_st    =>      wr_modem_st    ,
      flush          =>      flush          ,

      spi_sclk       =>      spi_sclk       ,
      spi_ss_n       =>      spi_ss_n       ,
      spi_miso       =>      spi_miso       ,
      spi_miosio_in  =>      spi_miosio_in  ,
      spi_miosio_out =>      spi_miosio_out ,
      spi_miosio_en  =>      spi_miosio_en  ,

      done_cmd       =>      done_cmd      ,
      rd             =>      rd            ,
      wr             =>      wr            ,
      data_out       =>      int_data_out      
   );

---------------------------------------------------------
-- byte counter for writes
---------------------------------------------------------

byte_count(15 downto 8)  <= data_in;
enable <= sel_data and rd;

bcnt1 : dncntlg 
   generic map(
      width => 16
   )
   port map(
      clk           => clk,
      enable        => enable,
      d             => byte_count,
      q             => act_count,
      load          => ld_cnt_high,
      async_reset_n => reset_n,
      sync_reset    => rst_bytecount
);

cnt_limit <= '1' when (act_count="0000000000000000") else '0';

---------------------------------------------------------
-- create state machine for spi interface
---------------------------------------------------------

spistp : process(reset_n,clk)
begin
if (reset_n='0') then
   spist        <= SPIDLE;
   spi_rd       <= '0';
   spi_wr       <= '0';
   rst_bytecount <= '0';
   sel_data     <= '0';
   ld_command   <= '0';
   ld_cnt_low   <= '0';
   ld_cnt_high  <= '0';
   spi_ld_setup <= '0';
   rd_modem_st  <= '0';
   wr_modem_st  <= '0';
   flush        <= '0';

elsif((clk'event) AND (clk='1')) then
   case spist is
--
   when SPIDLE =>
      spist <= SPIDLE;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';

--------------------------------------------
-- command
--------------------------------------------
   when SPCMD1 =>
      spist <= SPCMD2;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';
--
   when SPCMD2 =>
      if (command=CMD_SETUP) then
         spist <= SPSET1;
      elsif (command=CMD_RD_MDM) then
         spist <= SPRDM1;
      elsif (command=CMD_WR_MDM) then
         spist <= SPWRM1;
      elsif (command=CMD_FLUSH) then
         spist <= SPFLSH1;
      elsif (command=CMD_WRITE) then
         spist <= SPWR1;
      else
         spist <= SPIDLE;
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';

--------------------------------------------
-- setup
--------------------------------------------
   when SPSET1 =>
      if (data_av='1') then -- get setup data
         spist <= SPIDLE;
         spi_rd       <= '1';
         spi_ld_setup <= '1';
      else
         spist <= SPSET1;
         spi_rd       <= '0';
         spi_ld_setup <= '0';
      end if;
--      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
--      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';

--------------------------------------------
-- flush
--------------------------------------------
   when SPFLSH1 =>
      if (done_cmd='1') then 
         spist     <= SPIDLE;
         flush     <= '0';
      else
         spist     <= SPFLSH1;
         flush     <= '1';
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
--      flush        <= '0';

--------------------------------------------
-- wr modem stat
--------------------------------------------
   when SPWRM1 =>
      if (data_av='1') then -- modem data avail
         spist <= SPWRM2;
         wr_modem_st  <= '1';
         sel_data     <= '1';
      else
         spist <= SPWRM1;
         wr_modem_st  <= '0';
         sel_data     <= '0';
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
--      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
--      wr_modem_st  <= '0';
      flush        <= '0';
--
   when SPWRM2 =>
      if (rd='1') then -- get setup data
         spist <= SPIDLE;
         sel_data     <= '0';
         wr_modem_st  <= '0';
         spi_rd       <= '1';
      else
         spist <= SPWRM2;
         sel_data     <= '1';
         wr_modem_st  <= '1';
         spi_rd       <= '0';
      end if;
--      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
--      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
--      wr_modem_st  <= '0';
      flush        <= '0';

--------------------------------------------
-- rd modem stat
--------------------------------------------
   when SPRDM1 =>
      if (space_av='1') then -- modem data avail
         spist <= SPRDM2;
         rd_modem_st  <= '1';
      else
         spist <= SPRDM1;
         rd_modem_st  <= '0';
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
--      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';
--
   when SPRDM2 =>
      if (wr='1') then -- get setup data
         spist <= SPIDLE;
         rd_modem_st  <= '0';
      else
         spist <= SPRDM2;
         rd_modem_st  <= '1';
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
--      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';

--------------------------------------------
-- wr data
--------------------------------------------
   when SPWR1 =>
      if (data_av='1') then -- cnt low data avail
         spist <= SPWR2;
         ld_cnt_low   <= '1';
         spi_rd       <= '1';
      else
         spist <= SPWR1;
         ld_cnt_low   <= '0';
         spi_rd       <= '0';
      end if;
--      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '1';
      sel_data     <= '0';
      ld_command   <= '0';
--      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';
--
   when SPWR2 =>
      if (data_av='1') and (spi_rd='0') then -- cnt high data avail
         spist <= SPWR3;
         spi_rd       <= '1';
         ld_cnt_high  <= '1';
      else
         spist <= SPWR2;
         spi_rd       <= '0';
         ld_cnt_high  <= '0';
      end if;
--      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
--      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';
--
   when SPWR3 =>
      if (rd='1') and (cnt_limit='1') then -- wait for data to be consumed
         spist <= SPIDLE;
         sel_data     <= '0';
      else
         spist <= SPWR3;
         sel_data     <= '1';
      end if;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
--      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';


--------------------------------------------
-- others
--------------------------------------------

   when others =>
      spist        <= SPIDLE;
      spi_rd       <= '0';
      spi_wr       <= '0';
      rst_bytecount <= '0';
      sel_data     <= '0';
      ld_command   <= '0';
      ld_cnt_low   <= '0';
      ld_cnt_high  <= '0';
      spi_ld_setup <= '0';
      rd_modem_st  <= '0';
      wr_modem_st  <= '0';
      flush        <= '0';

   end case;
end if;
end process spistp;



---------------------------------------------------------
-- assign outputs
---------------------------------------------------------


ext_rd <= '0';
ext_wr <= '0';

end generate LOOPBACK_MODE;



end rtl;

