-------------------------------------------------------------------------------
-- Title      : MorphIC II wrapper for synchronous 245 Interface for testing FT2232H
-------------------------------------------------------------------------------
-- File       : MorphIC_HS_FT1248.vhd
-- Author     : AJ DOUGAN 
-- Company    : FTDI
-- Created    : 2010-04-20
-- Last update: 2010-04-20
-- Platform   : 
-- Standard   : VHDL'93
-------------------------------------------------------------------------------
-- Description: FT1248 master interface between FT232H and FT2232H device
-------------------------------------------------------------------------------
-- Copyright (c) 2010
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2010-04-20  1.0      AJD     Created
-------------------------------------------------------------------------------
--
-- HDATA0 - pin 13
-- HDATA1 - pin 14
-- HDATA2 - pin 15
-- HDATA3 - pin 16
-- HDATA4 - pin 19
-- HDATA5 - pin 20
-- HDATA6 - pin 21
-- HDATA7 - pin 22
-- 
-- HBDBUS0 - pin 23
-- HBDBUS1 - pin 26
-- HBDBUS2 - pin 27
-- HBDBUS3 - pin 28
-- HBDBUS4 - pin 68
-- HBDBUS5 - pin 69
-- HBDBUS6 - pin 70
-- HBDBUS7 - pin 71
-------------------------------------------------------------------------------


LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;

ENTITY MorphIC_HS_FT1248 IS
  PORT (-- Inputs
      clk50     : in  std_logic;       -- 50MHz clock input unused
      reset   : in  std_logic;       -- Active low reset via BD7

-- Morphic on board FT2232H signals
      mdata     : inout std_logic_vector(7 downto 0);   -- Port A Data Bus
      mrxfn     : in std_logic;
      mtxen     : in std_logic;
      mrdn      : out std_logic;
      mwrn      : out std_logic;
--      moen      : out std_logic;
--      old_clock : in std_logic;            -- unused
      msndimm   : out std_logic;           -- unused
--      mclk60    : in std_logic;
      

-- High speed Synchronous 245 signals
      acbus9    : in std_logic;            -- unused
      acbus8    : in std_logic;            -- unused
      acbus6    : in std_logic;            -- unused
      acbus5    : in std_logic;            -- unused
      acbus4    : in std_logic;            -- unused
      acbus3    : in std_logic;            -- unused
      hdata     : inout std_logic_vector(7 downto 0);
      sclk      : out std_logic;           -- SCLK
      ss_n      : out std_logic;           -- select #
      miso      : in  std_logic            -- miso
        );
END MorphIC_HS_FT1248;

ARCHITECTURE rtl OF MorphIC_HS_FT1248 IS

component FT1248_master_cont
   generic( 
      cpol_level : integer := 1;
      loopback   : integer := 0
   );
   port(
      reset_n        : in std_logic;
      clk            : in std_logic;
      data_in        : in std_logic_vector(7 downto 0);
      data_av        : in std_logic;
      space_av       : in std_logic;


-- outputs


      spi_sclk       : out std_logic;
      spi_ss_n       : out std_logic;
      spi_miso       : in  std_logic;
      spi_miosio_in  : in  std_logic_vector(7 downto 0);
      spi_miosio_out : out std_logic_vector(7 downto 0);
      spi_miosio_en  : out std_logic;

      ext_rd         : out std_logic;
      ext_wr         : out std_logic;
      data_out       : out std_logic_vector(7 downto 0)

);
end component;

component syncflop 
   generic ( G_NFLOPS : integer := 2 ) ;
   port (
      clk            :  in     std_logic;
      reset_n        :  in     std_logic;
      din            :  in     std_logic;
      dout           :  out    std_logic
   );
end component;

type asstates is (ASIDLE,ASRD1,ASRD2,ASRD3,ASRD4,ASRD5,ASRD6,ASWR1,ASWR2);

signal asst : asstates;

signal mdatain  : std_logic_vector(7 downto 0);
signal mdataout : std_logic_vector(7 downto 0);
signal mdataen  : std_logic;
signal mrxf,mtxe : std_logic;
signal mrxf1,mtxe1,m_rd,m_wr : std_logic;
signal set_rxf,clr_txf : std_logic;
  

signal 		reset_n : std_logic;
signal      spi_data_in    : std_logic_vector(7 downto 0);
signal      spi_data_av    : std_logic;
signal      spi_space_av   : std_logic;
signal      spi_sclk       : std_logic;
signal      spi_ss_n       : std_logic;
signal      spi_miso       : std_logic;
signal      spi_miosio_in  : std_logic_vector(7 downto 0);
signal      spi_miosio_out : std_logic_vector(7 downto 0);
signal      spi_miosio_en  : std_logic;
signal      spi_rd         : std_logic;
signal      spi_wr         : std_logic;
signal      spi_data_out   : std_logic_vector(7 downto 0);


BEGIN

--reset_n <= reset;   -- polarity to programme over JTAG  
reset_n <= not reset;   -- polarity to programme over USB

--============================================================
--== Create buffers for all Bidi I/Os for Morphic FT2232H side
--============================================================

mdataen <= not m_rd;

GEN_MORPHIC_DATABUS:

for I in 0 to 7 generate

mdbus_I : process(mdataout,mdataen)
begin
if (mdataen='1') then
   mdata(I) <= mdataout(I);
else
   mdata(I) <= 'Z';
end if;
end process mdbus_I;

mdatain(i) <= mdata(i);

end generate GEN_MORPHIC_DATABUS;

--============================================================
--== Create 245 interface async
--============================================================

syncp : process(reset_n,clk50)
begin
if(reset_n='0') then
   mrxf  <= '0';
   mrxf1 <= '0';
   mtxe  <= '0';
   mtxe1 <= '0';
elsif rising_edge(clk50) then
   mrxf1 <= not mrxfn and not m_rd;
   mrxf  <= mrxf1 and not m_rd;
   mtxe1 <= not mtxen;
   mtxe  <= mtxe1;
end if;
end process syncp;


-- create data buffer from spi to morphic
bds2m : process(reset_n,clk50)
begin
if(reset_n='0') then
   mdataout <= (others=>'0');
   spi_space_av <= '1';
elsif rising_edge(clk50) then
   if (spi_wr='1') then
      mdataout <= spi_data_out;
   end if;
   if (spi_wr='1') then
      spi_space_av <= '0';
   elsif (clr_txf='1') then
      spi_space_av <= '1';
   end if;
end if;
end process bds2m;


-- create data buffer from morphic to spi
bdm2s : process(reset_n,clk50)
begin
if(reset_n='0') then
   spi_data_in <= (others=>'0');
   spi_data_av <= '0';
elsif rising_edge(clk50) then
   if (set_rxf='1') then
      spi_data_in <= mdatain;
   end if;
   if (set_rxf='1') then
      spi_data_av <= '1';
   elsif (spi_rd='1') then
      spi_data_av <= '0';
   end if;
end if;
end process bdm2s;


---------------------------------------------------------
-- create state machine for 245 interface
---------------------------------------------------------

spistp : process(reset_n,clk50)
begin
if (reset_n='0') then
   asst       <= ASIDLE;
   m_rd       <= '0';
   m_wr       <= '0';
   set_rxf    <= '0';
   clr_txf    <= '0';

elsif((clk50'event) AND (clk50='1')) then
   case asst is
--
   when ASIDLE =>
      if (mrxf='1') and (m_rd='0') and ((spi_data_av='0') or (spi_rd='1')) then -- get data
         asst  <= ASRD1;
         m_rd  <= '1';
      elsif (spi_space_av='0') and (mtxe='1') and (m_rd='0') and (clr_txf='0') then -- write data
         asst  <= ASWR1;
         m_rd  <= '0';
      else
         asst  <= ASIDLE;
         m_rd  <= '0';
      end if;
--      m_rd       <= '0';
      m_wr       <= '0';
      set_rxf    <= '0';
      clr_txf    <= '0';

--------------------------------------------
-- read
--------------------------------------------
   when ASRD1 =>
      asst  <= ASRD2;
      m_rd       <= '1';
      m_wr       <= '0';
      set_rxf    <= '0';
      clr_txf    <= '0';
   when ASRD2 =>
      asst  <= ASRD3;
      m_rd       <= '1';
      m_wr       <= '0';
      set_rxf    <= '1';
      clr_txf    <= '0';
   when ASRD3 =>
      asst  <= ASRD4;
      m_rd       <= '0';
      m_wr       <= '0';
      set_rxf    <= '0';
      clr_txf    <= '0';
   when ASRD4 =>
      asst  <= ASRD5;
      m_rd       <= '0';
      m_wr       <= '0';
      set_rxf    <= '0';
      clr_txf    <= '0';
   when ASRD5 =>
      asst  <= ASRD6;
      m_rd       <= '0';
      m_wr       <= '0';
      set_rxf    <= '0';
      clr_txf    <= '0';
   when ASRD6 =>
      asst  <= ASIDLE;
      m_rd       <= '0';
      m_wr       <= '0';
      set_rxf    <= '0';
      clr_txf    <= '0';
--------------------------------------------
-- write
--------------------------------------------
   when ASWR1 =>
      asst  <= ASWR2;
      m_rd       <= '0';
      m_wr       <= '1';
      set_rxf    <= '0';
      clr_txf    <= '0';
   when ASWR2 =>
      asst  <= ASIDLE;
      m_rd       <= '0';
      m_wr       <= '0';
      set_rxf    <= '0';
      clr_txf    <= '1';

--------------------------------------------
-- others
--------------------------------------------

   when others =>
      asst       <= ASIDLE;
      m_rd       <= '0';
      m_wr       <= '0';
      set_rxf    <= '0';
      clr_txf    <= '0';
   end case;
end if;
end process spistp;




--========================================================
--== Create buffers for all Bidi I/Os for Target side
--========================================================


GEN_EXT_DATABUS:
for I in 0 to 7 generate

hdbus_I : process(spi_miosio_out,spi_miosio_en)
begin
if (spi_miosio_en='1') then
   hdata(I) <= spi_miosio_out(I);
else
   hdata(I) <= 'Z';
end if;
end process hdbus_I;
spi_miosio_in(i) <= hdata(i);
end generate GEN_EXT_DATABUS;

sclk     <= spi_sclk;
ss_n     <= spi_ss_n;
spi_miso <= miso;


--------------------------------------------------------------------------------
-- Instantiate fast serial control block
--------------------------------------------------------------------------------


spimast1 : FT1248_master_cont
   generic map( 
      cpol_level => 0,
      loopback   => 0

   )
   port map(
      reset_n        =>      reset_n        ,
      clk            =>      clk50          ,
      data_in        =>      spi_data_in    ,
      data_av        =>      spi_data_av    ,
      space_av       =>      spi_space_av   ,
      spi_sclk       =>      spi_sclk       ,
      spi_ss_n       =>      spi_ss_n       ,
      spi_miso       =>      spi_miso       ,
      spi_miosio_in  =>      spi_miosio_in  ,
      spi_miosio_out =>      spi_miosio_out ,
      spi_miosio_en  =>      spi_miosio_en  ,
      ext_rd         =>      spi_rd         ,
      ext_wr         =>      spi_wr         ,
      data_out       =>      spi_data_out       

);




mrdn <= not m_rd;
mwrn <= m_wr;

msndimm <= '1';




END rtl;

